#include <xc.h>
#include <pic16f886.h>
#include "port_init.h"
#include "user_define.h"
#include "eeprom.h"
#include "wait_time.h"
#include "m6955_am.h"
#include "usart.h"
#include "hitec_c_i2c.h"
#include "lm48100.h"

#pragma config BOREN = OFF
#pragma config CPD = OFF 
#pragma config FOSC =INTRC_NOCLKOUT  
#pragma config MCLRE = ON 
#pragma config WDTE = OFF 
#pragma config CP = OFF 
#pragma config LVP = OFF 
#pragma config PWRTE = ON
#pragma config IESO =OFF
#pragma config FCMEN = OFF

#pragma config BOR4V = BOR21V
#pragma config WRT = OFF


unsigned int	khz_no;
unsigned char   volume;

unsigned int	receive_flg;
unsigned char   rx_data[10];
unsigned char   rx_count;
unsigned int    rx_time_over;

unsigned char   mute_flg;
unsigned char   power_flg;
unsigned char   off_timer_flg;
unsigned char   time_sec;
unsigned char   time_min;
unsigned char   off_timer_sec;

unsigned char   timer_count;
unsigned char   timer_flg;

unsigned char   rep_count;
unsigned char   rep_flg;

/*--------------------USART INTERRUPT-------------*/
void	interrupt InterReceiver(void)
{
	if(RCIF==1){
		rx_data[rx_count]=RCREG;
        rx_count++;
        if(rx_count>4){
            receive_flg=1;
            rx_count=0;
        }
        rx_time_over=1;
        RB7=1;
		RCIF =0;

	}
    
    if (TMR1IF == 1) {
        timer_count++;
        if (timer_count >= 5) {
            timer_count = 0 ;
            timer_flg = 1;
        }
        if(rep_count>0){
            rep_count++;
            if(rep_count>REP_COUNT_INIT){
                rep_flg=1;
            }
        }
        TMR1L=TIMER1L_INIT;
        TMR1H=TIMER1H_INIT;
        TMR1IF=0;
     }
}

/*--------------------- Bluetooth init -----------*/
void    bluetooth_init(void)
{
    putch(0x02);
    putch(0x52);
    putch(0x4e);
    putch(0x01);
    putch(0x00);
    putch(0xa1);
    putch(0x03);
    putch(0x03);
}

/*---------------------UART TX--------------------*/
void	data_tx(unsigned int data)
{
unsigned char	no;

	no=(unsigned char)(data);
	putch(no);
}

void    tx_khz_no(unsigned int khz)
{
unsigned char n0,n1,n2,n3;
unsigned int count;

	count=khz;
	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);count=(count-n1)/10;
	n2=(unsigned char)(count%10);count=(count-n2)/10;
	n3=(unsigned char)(count%10);

    putch(0x46);        // F
    putch(0x3a);        // :
    putch(0x30+n3);
    putch(0x30+n2);
    putch(0x30+n1);
    putch(0x30+n0);
    putch(0x6b);        // k
    putch(0x48);        // H
    putch(0x7a);        // z
    //putch(0x0a);
    putch(0x0d);
}

void    tx_volume(unsigned char vol)
{
unsigned char n0,n1;
unsigned int count;

	count=vol;
	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);

    putch(0x56);        // V
    putch(0x4f);        // O
    putch(0x4c);        // L
    putch(0x3a);        // :
    putch(0x30+n1);
    putch(0x30+n0);
    //putch(0x0a);
    putch(0x0d);    
}

void    tx_power_on(void)
{
    putch(0x50);        // P
    putch(0x3a);        // :
    putch(0x4f);        // O
    putch(0x4e);        // N
    //putch(0x0a);
    putch(0x0d);      
}

void    tx_power_off(void)
{
    putch(0x50);        // P
    putch(0x3a);        // :
    putch(0x4f);        // O   
    putch(0x46);        // F
    putch(0x46);        // F
    //putch(0x0a);
    putch(0x0d);      
}

void    tx_signal(unsigned char sig)
{
unsigned char n0,n1;
unsigned int count;

	count=sig;
	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);

    putch(0x53);        // S
    putch(0x49);        // I
    putch(0x47);        // G
    putch(0x3a);        // :
    putch(0x30+n1);
    putch(0x30+n0);
    //putch(0x0a);
    putch(0x0d);    
}

void    tx_off_timer(unsigned char  min)
{
unsigned char n0,n1;
unsigned int count;

	count=min;
	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);

    putch(0x4f);        // O
    putch(0x46);        // F
    putch(0x46);        // F
    putch(0x3a);        // :
    putch(0x30+n1);
    putch(0x30+n0);
    putch(0x6d);        // m
    putch(0x69);        // i
    putch(0x6e);        // n
    //putch(0x0a);
    putch(0x0d);
}
/*---------------------SUJI HENKAN ---------------*/
unsigned int    suji_henkan(unsigned char *data)
{
unsigned int    no;

    no=(data[1]-0x30)*1000+(data[2]-0x30)*100+(data[3]-0x30)*10+(data[4]-0x30);
    
    return  no;
}

/*---------------------kHz SOUSA -----------------*/
unsigned int    khz_no_up(unsigned int khz)
{
unsigned int khz_ret;
    
    khz_ret=((khz/9)+1)*9;
    if(khz_ret>FREQ_MAX){
        khz_ret=FREQ_MAX;
    }
    
    return khz_ret;
}

unsigned int    khz_no_down(unsigned int khz)
{
unsigned int khz_ret;

    khz_ret=((khz/9)-1)*9;
    if(khz_ret<FREQ_MIN){
        khz_ret=FREQ_MIN;
    }
    
    return khz_ret;
}

/*--------------------- VOL SOUSA ----------------*/
unsigned char   volume_up(unsigned char vol)
{
unsigned char vol_ret;

    vol_ret=vol;
    if(vol_ret<VOLUME_MAX){
        vol_ret++;
    }
    
    return vol_ret;
}

unsigned char   volume_down(unsigned char vol)
{
unsigned char vol_ret;

    vol_ret=vol;
    if(vol_ret>VOLUME_MIN){
        vol_ret--;
    }
    
    return vol_ret;
}
/*-------------------Timer1-----------------------*/
void    timer1_init(void)
{
    T1CON=0x20;                 // 1:4
    TMR1L=TIMER1L_INIT;
    TMR1H=TIMER1H_INIT;
    TMR1IF=0;
    TMR1IE=1;
    PEIE=1;
    GIE=1;
    TMR1ON=1;
}
/*---------------------MAIN-----------------------*/
void	main(void)
{

	port_init();
    M6955_PON=1;NOP(); 
    
	__delay_ms(5000);

    khz_no=KHZ_INIT;
    volume=VOL_INIT;
    mute_flg=0;
    power_flg=1;
    off_timer_flg=0;
    timer_count=0;
    
    if(EEPROM_INIT==0){
        eeprom_check();
    }
    else{
        eeprom_init();
    }
        
	I2CInit();

	receive_flg=0;
    rx_count=0;
    rx_time_over=0;
	init_usart();	

    bluetooth_init();
    m6955_am_init();      
    lm48100_init();
    
	__delay_ms(3000);
   
 	tune_sub(khz_no);
    lm48100_vol_set(volume);

    timer1_init();
    
    rep_count=0;
    rep_flg=0;
    
	while(1){


		wait_time(1000);
		if(receive_flg==1){
             switch(rx_data[0]){
                case 'F':   khz_no=suji_henkan(rx_data);
                            tune_sub(khz_no);
                            tx_khz_no(khz_no);
                            eeprom_write_direct();
                            break;
                case 'G':   khz_no=khz_no_up(khz_no);
                            tune_sub(khz_no);
                            tx_khz_no(khz_no);
                            eeprom_write_direct();
                            break;
                case 'H':   khz_no=khz_no_down(khz_no);
                            tune_sub(khz_no);
                            tx_khz_no(khz_no);
                            eeprom_write_direct();
                            break;
                case 'R':   tx_khz_no(khz_no);
                            break;
                case 'M':   switch(mute_flg){
                                case 1: lm48100_vol_set(volume);
                                        tx_volume(volume);
                                        mute_flg=0;
                                        break;
                                case 0: lm48100_vol_set(0);
                                        tx_volume(0);
                                        mute_flg=1;
                                        break;
                            }
                            rep_count=1;
                            break;
                case 'P':   switch(power_flg){
                                case 0: power_flg=1;
                                        M6955_PON=1;NOP();
                                        __delay_ms(100);
                                        m6955_am_init();
                                        lm48100_init();
                                        tune_sub(khz_no);
                                        lm48100_vol_set(volume);
                                        tx_power_on();
                                        off_timer_flg=0;
                                        timer_count=0;
                                        rep_count=1;
                                        break;
                                case 1: power_flg=0;
                                        lm48100_vol_set(0);
                                        M6955_PON=0;NOP();
                                        lm48100_off();
                                        tx_power_off();
                                        break;
                            }
                            break;
                case 'V':   volume=suji_henkan(rx_data);
                            if(volume>31){
                                volume=31;
                            }
                            lm48100_vol_set(volume);
                            tx_volume(volume);
                            eeprom_write_direct();
                            rep_count=1;
                            break;
                case 'W':   volume=volume_up(volume);
                            lm48100_vol_set(volume);
                            tx_volume(volume);
                            eeprom_write_direct();
                            rep_count=1;
                            break;
                case 'X':   volume=volume_down(volume);
                            lm48100_vol_set(volume);
                            tx_volume(volume);
                            eeprom_write_direct();
                            rep_count=1;
                            break;
                 case 'T':  switch(off_timer_flg){
                                case 0: off_timer_flg=1;
                                        off_timer_sec=suji_henkan(rx_data);
                                        if(off_timer_sec>60){
                                            off_timer_sec=60;
                                        }
                                        tx_off_timer(off_timer_sec);
                                        time_sec=0;
                                        time_min=0;
                                        timer_count=0;
                                        break;
                                case 1: off_timer_flg=0;
                                        off_timer_sec=0;
                                        tx_off_timer(off_timer_sec);
                                        break;
                            }
                            rep_count=1;
                            break;
            }
			receive_flg=0;
		}

        if(rx_time_over!=0){
            rx_time_over++;
            if(rx_time_over>RX_INT_OVER_TIME){
                rx_time_over=0;
                rx_count=0;
                RB7=0;
            }
        }

        if((power_flg==1) && (rep_flg==1)){
            rep_count=0;
            rep_flg=0;
            tx_khz_no(khz_no);            
        }        
        
        if((off_timer_flg==1) && (timer_flg==1)){
            timer_flg=0;
            time_sec++;
            if(time_sec>=60){
                time_sec=0;
                time_min++;
                if(time_min>=off_timer_sec){
                    power_flg=0;
                    lm48100_vol_set(0);
                    M6955_PON=0;NOP();
                    lm48100_off();
                    off_timer_flg=0;
                    off_timer_sec=0;
                }
            }
        }
        
        
        
	}

}